@@ -1,4 +1,25 @@
+#!/usr/bin/python
+# -*- coding: utf-8 -*-
+# Copyright 2019 Silvio Peroni <essepuntato@gmail.com>
+# Copyright 2019-2020 Fabio Mariani <fabio.mariani555@gmail.com>
+# Copyright 2021 Simone Persiani <iosonopersia@gmail.com>
+# Copyright 2021-2022 Arcangelo Massari <arcangelo.massari@unibo.it>
+#
+# Permission to use, copy, modify, and/or distribute this software for any purpose
+# with or without fee is hereby granted, provided that the above copyright notice
+# and this permission notice appear in all copies.
+#
+# THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
+# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
+# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
+# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
+# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
+# SOFTWARE.
+
+
 from filelock import FileLock
+from oc_meta.constants import CONTAINER_EDITOR_TYPES
 from oc_meta.lib.cleaner import Cleaner
 from oc_meta.lib.csvmanager import CSVManager
 from oc_meta.lib.finder import *
@@ -275,11 +296,12 @@ class Curator:
         :returns: None -- This method modifies self.ardict, self.radict, and self.idra, and returns None.
         '''
         if row[col_name]:
-            if row['id'] in self.brdict or row['id'] in self.conflict_br:
-                br_metaval = row['id']
+            br_metaval_to_check = get_edited_br_metaid(row, self.vvi, row['id'], row['venue']) if col_name == 'editor' else row['id']
+            if br_metaval_to_check in self.brdict or br_metaval_to_check in self.conflict_br:
+                br_metaval = br_metaval_to_check
             else:
-                other_id = [id for id in self.brdict if row['id'] in self.brdict[id]['others']]
-                conflict_id = [id for id in self.conflict_br if row['id'] in self.conflict_br[id]['others']]
+                other_id = [id for id in self.brdict if br_metaval_to_check in self.brdict[id]['others']]
+                conflict_id = [id for id in self.conflict_br if br_metaval_to_check in self.conflict_br[id]['others']]
                 br_metaval = other_id[0] if other_id else conflict_id[0]
             if br_metaval not in self.ardict or not self.ardict[br_metaval][col_name]:
                 # new sequence
@@ -340,10 +362,11 @@ class Curator:
             ra_list = Cleaner.clean_ra_list(ra_list)
             for pos, ra in enumerate(ra_list):
                 new_elem_seq = True
-                ra_id = re.search(name_and_ids, ra)
-                if ra_id:
-                    name = Cleaner(ra_id.group(1)).clean_name()
-                    ra_id = ra_id.group(2)
+                ra_id = None
+                ra_id_match = re.search(name_and_ids, ra)
+                if ra_id_match:
+                    name = Cleaner(ra_id_match.group(1)).clean_name()
+                    ra_id = ra_id_match.group(2)
                 else:
                     name = Cleaner(ra).clean_name()
                 if not ra_id and sequence:
@@ -551,8 +574,8 @@ class Curator:
 
     def meta_maker(self):
         '''
-        For each dictionary ('brdict', 'ardict', 'radict') the corresponding MetaID dictionary is created
-        ('brmeta', 'armeta', and 'rameta').
+        For each dictionary ('brdict', 'ardict', 'radict', 'vvi') the corresponding MetaID dictionary is created
+        ('brmeta', 'armeta', 'rameta', and 'vvi').
         '''
         for identifier in self.brdict:
             if 'wannabe' in identifier:
@@ -591,6 +614,39 @@ class Curator:
             self.__meta_ar(br_key, ar_id, 'author')
             self.__meta_ar(br_key, ar_id, 'editor')
             self.__meta_ar(br_key, ar_id, 'publisher')
+        self.VolIss = dict()
+        if self.vvi:
+            for venue_meta in self.vvi:
+                venue_issue = self.vvi[venue_meta]['issue']
+                if venue_issue:
+                    for issue in venue_issue:
+                        issue_id = venue_issue[issue]['id']
+                        if 'wannabe' in issue_id:
+                            for br_meta in self.brmeta:
+                                if issue_id in self.brmeta[br_meta]['others']:
+                                    self.vvi[venue_meta]['issue'][issue]['id'] = str(br_meta)
+                venue_volume = self.vvi[venue_meta]['volume']
+                if venue_volume:
+                    for volume in venue_volume:
+                        volume_id = venue_volume[volume]['id']
+                        if 'wannabe' in volume_id:
+                            for br_meta in self.brmeta:
+                                if volume_id in self.brmeta[br_meta]['others']:
+                                    self.vvi[venue_meta]['volume'][volume]['id'] = str(br_meta)
+                        if venue_volume[volume]['issue']:
+                            volume_issue = venue_volume[volume]['issue']
+                            for issue in volume_issue:
+                                volume_issue_id = volume_issue[issue]['id']
+                                if 'wannabe' in volume_issue_id:
+                                    for br_meta in self.brmeta:
+                                        if volume_issue_id in self.brmeta[br_meta]['others']:
+                                            self.vvi[venue_meta]['volume'][volume]['issue'][issue]['id'] = str(br_meta)
+                if 'wannabe' in venue_meta:
+                    for br_meta in self.brmeta:
+                        if venue_meta in self.brmeta[br_meta]['others']:
+                            self.__merge_VolIss_with_vvi(br_meta, venue_meta)
+                else:
+                    self.__merge_VolIss_with_vvi(venue_meta, venue_meta)
 
     def enrich(self):
         '''
@@ -616,20 +672,22 @@ class Curator:
                     row['page'] = page
             elif metaid in self.remeta:
                 row['page'] = self.remeta[metaid][1]
-            self.ra_update(row, metaid, 'author')
-            self.ra_update(row, metaid, 'publisher')
-            self.ra_update(row, metaid, 'editor')
             row['id'] = ' '.join(self.brmeta[metaid]['ids'])
             row['title'] = self.brmeta[metaid]['title']
+            venue_metaid = None
             if row['venue']:
                 venue = row['venue']
                 if 'wannabe' in venue:
                     for i in self.brmeta:
                         if venue in self.brmeta[i]['others']:
-                            ve = i
+                            venue_metaid = i
                 else:
-                    ve = venue
-                row['venue'] = self.brmeta[ve]['title'] + ' [' + ' '.join(self.brmeta[ve]['ids']) + ']'
+                    venue_metaid = venue
+                row['venue'] = self.brmeta[venue_metaid]['title'] + ' [' + ' '.join(self.brmeta[venue_metaid]['ids']) + ']'
+            br_key_for_editor = get_edited_br_metaid(row, self.VolIss, metaid, venue_metaid)
+            self.ra_update(row, metaid, 'author')
+            self.ra_update(row, metaid, 'publisher')
+            self.ra_update(row, br_key_for_editor, 'editor')
 
     @staticmethod
     def name_check(ts_name, name):
@@ -738,40 +796,6 @@ class Curator:
             row['br'] = ''
             row['re'] = ''
             self.re_index.append(row)
-        # VI
-        self.VolIss = dict()
-        if self.vvi:
-            for venue_meta in self.vvi:
-                venue_issue = self.vvi[venue_meta]['issue']
-                if venue_issue:
-                    for issue in venue_issue:
-                        issue_id = venue_issue[issue]['id']
-                        if 'wannabe' in issue_id:
-                            for br_meta in self.brmeta:
-                                if issue_id in self.brmeta[br_meta]['others']:
-                                    self.vvi[venue_meta]['issue'][issue]['id'] = str(br_meta)
-                venue_volume = self.vvi[venue_meta]['volume']
-                if venue_volume:
-                    for volume in venue_volume:
-                        volume_id = venue_volume[volume]['id']
-                        if 'wannabe' in volume_id:
-                            for br_meta in self.brmeta:
-                                if volume_id in self.brmeta[br_meta]['others']:
-                                    self.vvi[venue_meta]['volume'][volume]['id'] = str(br_meta)
-                        if venue_volume[volume]['issue']:
-                            volume_issue = venue_volume[volume]['issue']
-                            for issue in volume_issue:
-                                volume_issue_id = volume_issue[issue]['id']
-                                if 'wannabe' in volume_issue_id:
-                                    for br_meta in self.brmeta:
-                                        if volume_issue_id in self.brmeta[br_meta]['others']:
-                                            self.vvi[venue_meta]['volume'][volume]['issue'][issue]['id'] = str(br_meta)
-                if 'wannabe' in venue_meta:
-                    for br_meta in self.brmeta:
-                        if venue_meta in self.brmeta[br_meta]['others']:
-                            self.__merge_VolIss_with_vvi(br_meta, venue_meta)
-                else:
-                    self.__merge_VolIss_with_vvi(venue_meta, venue_meta)
         if self.filename:
             if not os.path.exists(path_index):
                 os.makedirs(path_index)
@@ -1162,4 +1186,24 @@ class Curator:
             is_a_valid_row = True if br_venue and (br_volume or br_title) else False
         elif br_type == 'journal issue':
             is_a_valid_row = True if br_venue and (br_issue or br_title) else False
-        return is_a_valid_row
\ No newline at end of file
+        return is_a_valid_row
+
+def get_edited_br_metaid(row:dict, vvi_index:dict, metaid:str, venue_metaid:str):
+    if row['author'] and row['venue'] and row['type'] in CONTAINER_EDITOR_TYPES:
+        vol = row['volume']
+        issue = row['issue']
+        if vol:
+            vol_meta = vvi_index[venue_metaid]['volume'][vol]['id']
+        if issue:
+            if vol:
+                issue_meta = vvi_index[venue_metaid]['volume'][vol]['issue'][issue]['id']
+            else:
+                issue_meta = vvi_index[venue_metaid]['issue'][issue]['id']
+        edited_br_metaid = \
+            venue_metaid if row['type'] != 'journal article' \
+            else issue_meta if row['issue'] \
+            else vol_meta if row['volume'] \
+            else metaid
+    else:
+        edited_br_metaid = metaid
+    return edited_br_metaid
\ No newline at end of file
